/**
 * @file
 * Integrates Views data with FullCalendar.
 */
(function (Drupal, drupalSettings) {
  Drupal.behaviors.fullcalendar = {
    attach(context, settings) {
      async function pushUpdate(data) {
        const url = `${drupalSettings.path.baseUrl}fullcalendar/ajax/update/drop/${data.entity_type}/${data.id}?_format=json`;
        delete data.entity_type;
        delete data.id;
        try {
          const response = await fetch(url, {
            method: 'POST',
            headers: {
              'Content-Type': 'application/json',
            },
            body: JSON.stringify(data),
          });
          if (!response.ok) {
            throw new Error(`Response status: ${response.status}`);
          }

          // @todo display messages returned to the user.
          const json = await response.json();
          if (response.ok) {
            return true;
          }
          console.log(json);
        } catch (error) {
          console.error(error.message);
        }
        return false;
      }

      // Event drop call back function.
      function eventDrop(info) {
        const end = info.event.end;
        const start = info.event.start;
        let strEnd = '';
        let strStart = '';
        const viewIndex = this.el.id;
        const viewSettings = drupalSettings.fullcalendar[viewIndex];
        const formatSettings = {
          month: '2-digit',
          year: 'numeric',
          day: '2-digit',
          hour: '2-digit',
          minute: '2-digit',
          second: '2-digit',
        };
        // define the end date string in 'YYYY-MM-DD' format.
        if (end) {
          // The end date of an all-day event is exclusive. For example, the end of
          // 2018-09-03 will appear to 2018-09-02 in the calendar. So we need one day
          // subtract to ensure the day stored in Drupal is the same as when it
          // appears in the calendar.
          if (
            end.getUTCHours() === 0 &&
            end.getUTCMinutes() === 0 &&
            end.getUTCSeconds() === 0
          ) {
            end.setDate(end.getDate() - 1);
          }
          // String of the end date.
          // eslint-disable-next-line no-undef
          strEnd = FullCalendar.formatDate(end, formatSettings);
        }
        // define the start date string in 'YYYY-MM-DD' format.
        if (start) {
          // eslint-disable-next-line no-undef
          strStart = FullCalendar.formatDate(start, formatSettings);
        }

        if (viewSettings.updateConfirm === 1) {
          const title = info.event.title.replace(/(<([^>]+)>)/gi, '');
          const msg = Drupal.t(
            '@title start is now @event_start and end is now @event_end - Do you want to save this change?',
            {
              '@title': title,
              '@event_start': strStart,
              '@event_end': strEnd,
            },
          );

          // eslint-disable-next-line no-restricted-globals
          if (!confirm(msg)) {
            info.revert();
            return;
          }
        }

        const data = {
          id: info.event.id,
          eid: info.event.extendedProps.eid,
          entity_type: viewSettings.entityType,
          start: strStart,
          end: strEnd,
          allDay: info.event.allDay,
          startField: info.event.extendedProps.startField,
          endField: info.event.extendedProps.endField,
          type: info.event.extendedProps.type,
          token: viewSettings.token,
        };
        if (!pushUpdate(data)) {
          info.revert();
        }
      }

      if (document.addEventListener) {
        document.addEventListener('DOMContentLoaded', function () {
          Object.keys(drupalSettings.fullcalendar).forEach((domId) => {
            if (drupalSettings.fullcalendar.hasOwnProperty(domId)) {
              const calendarEl = document.getElementById(domId);
              const calendarOptions =
                drupalSettings.fullcalendar[domId].options;
              calendarOptions.eventDrop = eventDrop;
              // eslint-disable-next-line no-undef
              const calendar = new FullCalendar.Calendar(
                calendarEl,
                calendarOptions,
              );
              calendar.render();
            }
          });
        });
      }
    },
  };
})(Drupal, drupalSettings);
